@@ -12,58 +12,61 @@
 use std::{
     fs::File,
     os::fd::OwnedFd,
-    sync::{Arc, Condvar, Mutex},
+    sync::{Arc, Condvar, Mutex, RwLock},
 };
 
-use ahash::HashMapExt;
+use ahash::{HashMapExt, HashSetExt};
 use libseccomp::ScmpSyscall;
-use nix::{errno::Errno, unistd::Pid};
+use nix::{errno::Errno, sys::socket::UnixAddr, unistd::Pid};
 use serde::{ser::SerializeMap, Serializer};
 
 use crate::{
     confine::{ScmpNotifReq, SydArch, SydMemoryMap},
     elf::ExecutableFile,
-    hash::SydHashMap,
-    lookup::CanonicalPath,
+    hash::{SydHashMap, SydHashSet},
     sigset::SydSigSet,
 };
 
 /// Metadata on a blocking syscall invocation
 #[derive(Debug)]
-pub struct SysInterrupt {
+pub(crate) struct SysInterrupt {
     /// Syd handler thread ID
-    pub handler: Pid,
+    pub(crate) handler: Pid,
     /// System call request
-    pub request: ScmpNotifReq,
+    pub(crate) request: ScmpNotifReq,
     /// proc_pid_status(5) file handle
-    pub status: Option<OwnedFd>,
+    pub(crate) status: Option<OwnedFd>,
     /// Used by syd_emu to signal syd_int to delete the entry and close the file.
     /// This is because the status file descriptor is not valid in syd_emu's fs space.
-    pub delete: bool,
+    pub(crate) delete: bool,
+    /// Used by syd_mon to signal syd_int to signal stuck emulators manually,
+    /// when not enough resources are available to spawn new emulator threads.
+    /// This is because the status file descriptor is not valid in syd_mon's fs space.
+    pub(crate) signal: bool,
     /// True if `SA_RESTART` is ignored
     /// (e.g. due to a socket timeout).
-    pub ignore_restart: bool,
+    pub(crate) ignore_restart: bool,
 }
 
 /// Map of metadata on blocking syscall invocations.
-pub type BlockVec = Vec<SysInterrupt>;
+pub(crate) type BlockVec = Vec<SysInterrupt>;
 
 /// Map of restarting signals by TGID.
-pub type RestartMap = SydHashMap<Pid, SydSigSet>;
+pub(crate) type RestartMap = SydHashMap<Pid, SydSigSet>;
 
 /// This is the data type used to handle syscall interrupts.
 #[derive(Debug)]
-pub struct SysInterruptMap {
+pub(crate) struct SysInterruptMap {
     /// Map of blocking syscalls by request id.
-    pub sys_block: Arc<(Mutex<BlockVec>, Condvar)>,
+    pub(crate) sys_block: Arc<(Mutex<BlockVec>, Condvar)>,
     /// Map of restarting signals by TGID.
     /// Used for SA_RESTART tracking.
-    pub sig_restart: Arc<Mutex<RestartMap>>,
+    pub(crate) sig_restart: Arc<Mutex<RestartMap>>,
 }
 
 /// Represents an exec(3) check result
 #[derive(Debug)]
-pub struct ExecResult {
+pub(crate) struct ExecResult {
     pub(crate) exe: ExecutableFile,
     pub(crate) file: File,
     pub(crate) ip: u64,
@@ -75,35 +78,61 @@ pub struct ExecResult {
 }
 
 /// Syscall-agnostic error map.
-pub type ErrorMap = SydHashMap<Pid, Option<Errno>>;
+pub(crate) type ErrorMap = SydHashMap<Pid, Option<Errno>>;
 
-/// chdir(2) result map.
-pub type ChdirMap<'a> = SydHashMap<Pid, CanonicalPath<'a>>;
+/// chdir(2) result set.
+pub(crate) type ChdirSet = SydHashSet<Pid>;
 
 /// exec(3) result map.
-pub type ExecvMap = SydHashMap<Pid, ExecResult>;
+pub(crate) type ExecvMap = SydHashMap<Pid, ExecResult>;
+
+/// mmap(2) pid set.
+pub(crate) type MmapSet = SydHashSet<Pid>;
+
+// [inode,(pid,path)] map of unix binds.
+// Path is only used for UNIX domain sockets.
+//
+// SAFETY:
+// 1. /proc/net/unix only gives inode information,
+//    and does not include information on device id
+//    or mount id so unfortunately we cannot check
+//    for that here.
+// 2. Pid is used for SO_PEERCRED getsockopt(2).
+#[derive(Copy, Clone)]
+pub(crate) struct UnixVal {
+    pub(crate) pid: Pid,
+    pub(crate) addr: Option<UnixAddr>,
+    pub(crate) peer: Option<UnixAddr>,
+}
+pub(crate) type UnixMap = Arc<RwLock<SydHashMap<u64, UnixVal>>>;
+
+// [tid, tgid] map for ptrace(PTRACE_TRACEME) calling tids.
+// This is used to prevent ptrace(2) detection efficiently.
+pub(crate) type PtraceMap = Arc<RwLock<SydHashMap<Pid, Pid>>>;
 
 /// Results map for ptrace(2) hooks chdir, execve, sigaction and sigreturn.
 #[derive(Debug)]
-pub struct SysResultMap<'a> {
+pub(crate) struct SysResultMap {
     /// syscall-agnostic error map
-    pub trace_error: Arc<Mutex<ErrorMap>>,
+    pub(crate) trace_error: Arc<Mutex<ErrorMap>>,
     /// chdir(2) result map
-    pub trace_chdir: Arc<Mutex<ChdirMap<'a>>>,
+    pub(crate) trace_chdir: Arc<Mutex<ChdirSet>>,
     /// exec(3) result map
-    pub trace_execv: Arc<Mutex<ExecvMap>>,
+    pub(crate) trace_execv: Arc<Mutex<ExecvMap>>,
+    /// mmap(2) pid set.
+    pub(crate) trace_mmap: Arc<Mutex<MmapSet>>,
 }
 
 /// Map of TGIDs that have received count signals for handled signals.
-pub type SighandleMap = SydHashMap<Pid, u64>;
+pub(crate) type SighandleMap = SydHashMap<Pid, u64>;
 
 /// Signal map, used by signal counting for SROP mitigation:
 /// If a TGID is not in sig_handle_map at the entry of sigreturn(2),
 /// we terminate the process because the sigreturn(2) is artificial.
 #[derive(Debug)]
-pub struct SignalMap {
+pub(crate) struct SignalMap {
     /// Set of TGIDs that have received count signals for handled signals.
-    pub sig_handle: Arc<Mutex<SighandleMap>>,
+    pub(crate) sig_handle: Arc<Mutex<SighandleMap>>,
 }
 
 impl SysInterrupt {
@@ -118,6 +147,7 @@ impl SysInterrupt {
             ignore_restart,
             status: None,
             delete: false,
+            signal: false,
         })
     }
 
@@ -151,7 +181,7 @@ impl serde::Serialize for SysInterrupt {
             .unwrap_or_else(|_| format!("{}", i32::from(data.syscall)));
         let _ = map.serialize_entry("pid", &self.request.pid);
         let _ = map.serialize_entry("sys", &syscall);
-        let _ = map.serialize_entry("arch", &SydArch(data.arch));
+        let _ = map.serialize_entry("arch", &SydArch::from(data.arch));
         let _ = map.serialize_entry("args", &data.args);
         let _ = map.serialize_entry("handler", &self.handler.as_raw());
         let _ = map.serialize_entry("ignore_restart", &self.ignore_restart);
@@ -160,8 +190,18 @@ impl serde::Serialize for SysInterrupt {
     }
 }
 
+/// Create a new UnixMap.
+pub(crate) fn unix_map_new() -> UnixMap {
+    Arc::new(RwLock::new(SydHashMap::default()))
+}
+
+/// Create a new PtraceMap.
+pub(crate) fn ptrace_map_new() -> PtraceMap {
+    Arc::new(RwLock::new(SydHashMap::default()))
+}
+
 /// Create a new SysInterruptMap.
-pub fn sys_interrupt_map_new() -> SysInterruptMap {
+pub(crate) fn sys_interrupt_map_new() -> SysInterruptMap {
     SysInterruptMap {
         sys_block: Arc::new((Mutex::new(BlockVec::new()), Condvar::new())),
         sig_restart: Arc::new(Mutex::new(RestartMap::new())),
@@ -169,16 +209,17 @@ pub fn sys_interrupt_map_new() -> SysInterruptMap {
 }
 
 /// Create a new SysResultMap.
-pub fn sys_result_map_new<'a>() -> SysResultMap<'a> {
+pub(crate) fn sys_result_map_new() -> SysResultMap {
     SysResultMap {
         trace_error: Arc::new(Mutex::new(ErrorMap::new())),
-        trace_chdir: Arc::new(Mutex::new(ChdirMap::new())),
+        trace_chdir: Arc::new(Mutex::new(ChdirSet::new())),
         trace_execv: Arc::new(Mutex::new(ExecvMap::new())),
+        trace_mmap: Arc::new(Mutex::new(MmapSet::new())),
     }
 }
 
 /// Create a new SignalMap.
-pub fn signal_map_new() -> SignalMap {
+pub(crate) fn signal_map_new() -> SignalMap {
     SignalMap {
         sig_handle: Arc::new(Mutex::new(SighandleMap::new())),
     }
