@@ -170,7 +170,7 @@ fn syscall_chmod_handler(
             // Fallback to `/proc` indirection,
             //
             // path to fd is open already!
-            let pfd = XPathBuf::from_self_fd(fd.as_raw_fd());
+            let pfd = XPathBuf::from_self_fd(fd.as_raw_fd())?;
             pfd.with_nix_path(|cstr| {
                 // SAFETY: We deliberately bypass the libc wrapper here.
                 Errno::result(unsafe {
