@@ -14,7 +14,8 @@ use nix::errno::Errno;
 
 use crate::{
     compat::MFdFlags,
-    config::{HAVE_MFD_NOEXEC_SEAL, MFD_HUGETLB_NAME_PREFIX, MFD_NAME_PREFIX, MMAP_MIN_ADDR},
+    config::{HAVE_MFD_NOEXEC_SEAL, MFD_HUGETLB_NAME_PREFIX, MFD_NAME_PREFIX},
+    confine::is_valid_ptr,
     cookie::safe_memfd_create,
     kernel::sandbox_path,
     path::XPathBuf,
@@ -37,7 +38,7 @@ pub(crate) fn sys_memfd_create(request: UNotifyEventRequest) -> ScmpNotifResp {
         let mut flags = to_mfdflags(flags)?;
 
         // Validate name argument next.
-        if addr < *MMAP_MIN_ADDR {
+        if !is_valid_ptr(addr) {
             // Return EFAULT for invalid pointer.
             return Err(Errno::EFAULT);
         }
