@@ -135,6 +135,9 @@ pub(crate) mod xattr;
 /// ptrace(2) hooks
 pub(crate) mod ptrace;
 
+/// ptrace(2) syscall handler
+pub(crate) mod sys_ptrace;
+
 use libseccomp::ScmpNotifResp;
 use nix::{
     errno::Errno,
@@ -148,7 +151,6 @@ use nix::{
 
 use crate::{
     fs::{fd_status_flags, to_valid_fd},
-    ioctl::Ioctl,
     log::log_is_main,
     log_enabled,
     lookup::{CanonicalPath, FileInfo, FileType},
@@ -176,12 +178,15 @@ pub(crate) fn sandbox_path(
         return Err(if hide { Errno::ENOENT } else { Errno::EACCES });
     }
 
+    // Convert /proc/$pid to /proc/self as necessary.
+    let path = path.replace_proc_self(pid);
+
     // Check enabled capabilities.
     let caps_old = caps;
     let mut caps = sandbox.getcaps(caps);
     let stat = sandbox.enabled(Capability::CAP_STAT);
     if caps.is_empty() && (!hide || !stat) {
-        return if caps_old.can_write() && sandbox.is_append(path) {
+        return if caps_old.can_write() && sandbox.is_append(&path) {
             // SAFETY: Protect append-only paths against writes.
             // We use ECANCELED which will result in a no-op.
             Err(Errno::ECANCELED)
@@ -201,7 +206,7 @@ pub(crate) fn sandbox_path(
 
     // Sandboxing.
     for cap in check_caps {
-        let (new_action, new_filter) = sandbox.check_path(cap, path);
+        let (new_action, new_filter) = sandbox.check_path(cap, &path);
 
         if new_action >= action {
             action = new_action;
@@ -213,7 +218,7 @@ pub(crate) fn sandbox_path(
     // for sandboxing types other than Stat.
     let check_hidden = stat && hide && (caps.is_empty() || action.is_denying());
     if check_hidden || caps.contains(Capability::CAP_STAT) {
-        let (new_action, new_filter) = sandbox.check_path(Capability::CAP_STAT, path);
+        let (new_action, new_filter) = sandbox.check_path(Capability::CAP_STAT, &path);
 
         if !check_hidden {
             deny_errno = Errno::ENOENT;
@@ -250,20 +255,11 @@ pub(crate) fn sandbox_path(
         };
 
         if let Some(request) = request {
-            let arch = request.scmpreq.data.arch;
             let args = request.scmpreq.data.args;
-
-            // Enrich log data with ioctl(2) request names.
-            let ctl = if caps.contains(Capability::CAP_IOCTL) {
-                sandbox.get_ioctl_names(args[1] as Ioctl, arch)
-            } else {
-                None
-            };
-
             if sandbox.log_scmp() {
                 if is_warn {
                     warn!("ctx": "access", "cap": caps, "act": action,
-                        "sys": syscall_name, "ctl": ctl,
+                        "sys": syscall_name,
                         "path": &path, "args": args,
                         "tip": format!("configure `allow/{}+{}'",
                             caps.to_string().to_ascii_lowercase(),
@@ -271,7 +267,7 @@ pub(crate) fn sandbox_path(
                         "req": request);
                 } else {
                     notice!("ctx": "access", "cap": caps, "act": action,
-                        "sys": syscall_name, "ctl": ctl,
+                        "sys": syscall_name,
                         "path": &path, "args": args,
                         "tip": format!("configure `allow/{}+{}'",
                             caps.to_string().to_ascii_lowercase(),
@@ -280,7 +276,7 @@ pub(crate) fn sandbox_path(
                 }
             } else if is_warn {
                 warn!("ctx": "access", "cap": caps, "act": action,
-                    "sys": syscall_name, "ctl": ctl,
+                    "sys": syscall_name,
                     "path": &path, "args": args,
                     "tip": format!("configure `allow/{}+{}'",
                         caps.to_string().to_ascii_lowercase(),
@@ -288,7 +284,7 @@ pub(crate) fn sandbox_path(
                     "pid": request.scmpreq.pid);
             } else {
                 notice!("ctx": "access", "cap": caps, "act": action,
-                    "sys": syscall_name, "ctl": ctl,
+                    "sys": syscall_name,
                     "path": &path, "args": args,
                     "tip": format!("configure `allow/{}+{}'",
                         caps.to_string().to_ascii_lowercase(),
@@ -314,7 +310,7 @@ pub(crate) fn sandbox_path(
 
     match action {
         Action::Allow | Action::Warn => {
-            if caps.can_write() && sandbox.is_append(path) {
+            if caps.can_write() && sandbox.is_append(&path) {
                 // SAFETY: Protect append-only paths against writes.
                 // We use ECANCELED which will result in a no-op.
                 Err(Errno::ECANCELED)
@@ -390,8 +386,8 @@ where
         //
         // Delay Chdir to allow the common `cd /` use case right after chroot(2).
         let sandbox = request.get_sandbox();
-        if sandbox.is_chroot() && !caps.contains(Capability::CAP_CHDIR) {
-            return Err(if is_fd { Errno::EACCES } else { Errno::ENOENT });
+        if sandbox.is_chroot() && !is_fd && !caps.contains(Capability::CAP_CHDIR) {
+            return Err(Errno::ENOENT);
         }
 
         // If sandboxing for all the selected capabilities is off, return immediately.
@@ -441,7 +437,7 @@ where
                     // Handle ftruncate etc. for files with encryption in progress.
                     let crypt_path = if crypt {
                         #[expect(clippy::disallowed_methods)]
-                        let files = request.crypt_map.as_ref().unwrap();
+                        let files = request.cache.crypt_map.as_ref().unwrap();
                         if let Ok(info) = FileInfo::from_fd(&fd) {
                             let files = files.0.lock().unwrap_or_else(|e| e.into_inner());
                             files
