@@ -16,7 +16,7 @@ use nix::{
 
 use crate::{
     compat::getsockdomain,
-    config::MMAP_MIN_ADDR,
+    confine::is_valid_ptr,
     cookie::safe_accept4,
     fs::{get_nonblock, has_recv_timeout},
     kernel::net::sandbox_addr,
@@ -43,13 +43,13 @@ pub(crate) fn handle_accept(
     let addrlen = if args[2] != 0 {
         const SIZEOF_SOCKLEN_T: usize = size_of::<libc::socklen_t>();
         let mut buf = [0u8; SIZEOF_SOCKLEN_T];
-        if request.read_mem(&mut buf, args[2])? == SIZEOF_SOCKLEN_T {
+        if request.read_mem(&mut buf, args[2], SIZEOF_SOCKLEN_T)? == SIZEOF_SOCKLEN_T {
             // libc defines socklen_t as u32,
             // however we should check for negative values
             // and return EINVAL as necessary.
             let len = i32::from_ne_bytes(buf);
             let len = libc::socklen_t::try_from(len).or(Err(Errno::EINVAL))?;
-            if args[1] < *MMAP_MIN_ADDR {
+            if !is_valid_ptr(args[1]) {
                 // address length is positive however address is NULL:
                 // Return EINVAL and NOT EFAULT here, see LTP accept01 check.
                 return Err(Errno::EINVAL);
