@@ -8,7 +8,7 @@
 
 use std::{
     borrow::Cow,
-    net::IpAddr,
+    net::{IpAddr, Ipv4Addr, Ipv6Addr, SocketAddrV4, SocketAddrV6},
     ops::Deref,
     os::{
         fd::{AsRawFd, RawFd},
@@ -324,6 +324,8 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
         } // else we have a valid address to check for access.
 
         let sandbox = request.get_sandbox();
+        let anyaddr = sandbox.flags.allow_unsafe_any_addr();
+        let local_net = sandbox.flags.force_local_net();
         let argaddr = if op == 0x2 && addr_len as usize == size_of::<libc::sa_family_t>() {
             // unix(7): "If a bind(2) call specifies addrlen as `sizeof(sa_family_t)`,
             // or [...], then the socket is autobound to an abstract address".
@@ -334,12 +336,42 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
         } else {
             get_addr(&request, addr_remote, addr_len)?
         };
-        let (addr, root) = canon_addr(&request, &sandbox, &argaddr, cap)?;
+        let (mut addr, root) = canon_addr(&request, &sandbox, &argaddr, cap)?;
         match addr_family(&addr) {
-            PF_UNIX | PF_INET | PF_INET6 => {
+            PF_UNIX => {
                 // Check for access.
                 sandbox_addr(&request, &sandbox, &addr, &root, op, cap)?;
             }
+            PF_INET => {
+                // Turn 0.0.0.0 into 127.0.0.1 unless trace/allow_unsafe_any_addr:1.
+                // Turn all addresses to 127.0.0.1 if trace/force_local_net:1.
+                if !anyaddr {
+                    make_loaddr(&mut addr, local_net);
+                }
+                // Check for access.
+                sandbox_addr(&request, &sandbox, &addr, &root, op, cap).map_err(|errno| {
+                    if op == 0x2 /*bind*/ && has_privileged_port_v4(&addr) {
+                        Errno::EACCES
+                    } else {
+                        errno
+                    }
+                })?;
+            }
+            PF_INET6 => {
+                // Turn :: into :1 unless trace/allow_unsafe_any_addr:1.
+                // Turn all addresses to ::1 if trace/force_local_net:1.
+                if !anyaddr {
+                    make_lo6addr(&mut addr, local_net);
+                }
+                // Check for access.
+                sandbox_addr(&request, &sandbox, &addr, &root, op, cap).map_err(|errno| {
+                    if op == 0x2 /*bind*/ && has_privileged_port_v6(&addr) {
+                        Errno::EACCES
+                    } else {
+                        errno
+                    }
+                })?;
+            }
             PF_UNSPEC => {
                 // SAFETY: We do not check address for AF_UNSPEC:
                 //
@@ -633,28 +665,12 @@ pub(crate) fn sandbox_addr_unix(
         }
     };
 
-    // Convert /proc/${pid} to /proc/self as necessary.
-    let path = if let Some(p) = path.split_prefix(b"/proc") {
-        let mut buf = itoa::Buffer::new();
-        let req = request.scmpreq;
-        let pid = buf.format(req.pid);
-        if let Some(p) = p.split_prefix(pid.as_bytes()) {
-            let mut pdir = XPathBuf::from("/proc/self");
-            pdir.push(p.as_bytes());
-            Cow::Owned(pdir)
-        } else {
-            path
-        }
-    } else {
-        path
-    };
-
     // Check for access.
     let (action, filter) = sandbox.check_unix(caps, &path);
 
     if !filter {
         let sys = op2name(op);
-        if sandbox.verbose {
+        if sandbox.log_scmp() {
             warn!("ctx": "access", "cap": caps, "act": action,
                 "sys": sys, "unix": &path, "abs": abs,
                 "tip": format!("configure `allow/{caps}+{path}'"),
@@ -729,7 +745,7 @@ pub(crate) fn sandbox_addr_inet(
         // accept{,4}
         let ipv = if addr.is_ipv6() { 6 } else { 4 };
         let sys = op2name(op);
-        if sandbox.verbose {
+        if sandbox.log_scmp() {
             warn!("ctx": "block", "act": action,
                 "sys": sys, "addr": format!("{addr}!{port}"), "ipv": ipv,
                 "tip": format!("configure `block-{addr}'"),
@@ -744,7 +760,7 @@ pub(crate) fn sandbox_addr_inet(
         // connect, sendto, send{m,}msg
         let ipv = if addr.is_ipv6() { 6 } else { 4 };
         let sys = op2name(op);
-        if sandbox.verbose {
+        if sandbox.log_scmp() {
             warn!("ctx": "access", "cap": caps, "act": action,
                 "sys": sys, "addr": format!("{addr}!{port}"), "ipv": ipv,
                 "tip": format!("configure `allow/{caps}+{addr}!{port}'"),
@@ -806,7 +822,7 @@ pub(crate) fn sandbox_addr_unnamed(
 
     if !filter {
         let sys = op2name(op);
-        if sandbox.verbose {
+        if sandbox.log_scmp() {
             warn!("ctx": "access", "cap": caps, "act": action,
                 "sys": sys, "unix": &name, "abs": true,
                 "tip": format!("configure `allow/{caps}+{name}'"),
@@ -836,3 +852,68 @@ pub(crate) fn to_msgflags(arg: u64) -> Result<MsgFlags, Errno> {
     let flags = arg.try_into().or(Err(Errno::EINVAL))?;
     MsgFlags::from_bits(flags).ok_or(Errno::EINVAL)
 }
+
+// Change 0.0.0.0 to 127.0.0.1 (IPv4)
+// If `force` is true change all addresses.
+pub(crate) fn make_loaddr(addr: &mut SockaddrStorage, force: bool) {
+    if !force && !is_anyaddr(addr) {
+        return;
+    }
+    if let Some(sin) = addr.as_sockaddr_in() {
+        let port = sin.port();
+        *addr = SockaddrStorage::from(SocketAddrV4::new(Ipv4Addr::LOCALHOST, port));
+    }
+}
+
+// Change :: to ::1 (IPv6)
+// If `force` is true change all addresses.
+pub(crate) fn make_lo6addr(addr: &mut SockaddrStorage, force: bool) {
+    if !force && !is_any6addr(addr) {
+        return;
+    }
+    if let Some(sin6) = addr.as_sockaddr_in6() {
+        let port = sin6.port();
+        let flowinfo = sin6.flowinfo();
+        let scope_id = sin6.scope_id();
+        *addr = SockaddrStorage::from(SocketAddrV6::new(
+            Ipv6Addr::LOCALHOST,
+            port,
+            flowinfo,
+            scope_id,
+        ));
+    }
+}
+
+// Return true if `addr` is 0.0.0.0 (Ipv4).
+fn is_anyaddr(addr: &SockaddrStorage) -> bool {
+    addr.as_sockaddr_in()
+        .map(|sin| sin.ip() == Ipv4Addr::UNSPECIFIED)
+        .unwrap_or(false)
+}
+
+// Return true if `addr` is :: (Ipv6).
+fn is_any6addr(addr: &SockaddrStorage) -> bool {
+    addr.as_sockaddr_in6()
+        .map(|sin6| sin6.ip() == Ipv6Addr::UNSPECIFIED)
+        .unwrap_or(false)
+}
+
+// Return true if `addr` has a privileged port (< 1024) (Ipv4).
+fn has_privileged_port_v4(addr: &SockaddrStorage) -> bool {
+    addr.as_sockaddr_in()
+        .map(|sin| {
+            let port = sin.port();
+            port != 0 && port < 1024
+        })
+        .unwrap_or(false)
+}
+
+// Return true if `addr` has a privileged port (< 1024) (Ipv6).
+fn has_privileged_port_v6(addr: &SockaddrStorage) -> bool {
+    addr.as_sockaddr_in6()
+        .map(|sin6| {
+            let port = sin6.port();
+            port != 0 && port < 1024
+        })
+        .unwrap_or(false)
+}
