@@ -21,12 +21,13 @@ use zeroize::Zeroizing;
 use crate::{
     compat::{
         addr_family, cmsg_len_32, cmsg_space_32, msghdr, msghdr32, MsgFlags, PF_ALG, PF_INET,
-        PF_INET6, PF_NETLINK, PF_UNIX, UIO_MAXIOV, UIO_MAXLEN,
+        PF_INET6, PF_NETLINK, PF_UNIX, UIO_MAXIOV,
     },
+    config::MAX_RW_COUNT,
     confine::scmp_arch_bits,
     fs::{get_nonblock, has_recv_timeout, is_same_vm},
     kernel::{
-        net::{canon_addr, get_addr, sandbox_addr, to_msgflags},
+        net::{canon_addr, get_addr, make_lo6addr, make_loaddr, sandbox_addr, to_msgflags},
         sandbox_path,
     },
     path::XPath,
@@ -85,12 +86,46 @@ pub(crate) fn handle_sendmsg(
 
     // Step 1: Handle the address.
     let sandbox = request.get_sandbox();
-    let verbose = sandbox.verbose;
+    let any_addr = sandbox.flags.allow_unsafe_any_addr();
+    let local_net = sandbox.flags.force_local_net();
+    let log_scmp = sandbox.log_scmp();
     let addr = if !addr_remote.is_null() && addr_len > 0 {
         let addr = get_addr(request, addr_remote as u64, addr_len)?;
-        let (addr, root) = canon_addr(request, &sandbox, &addr, Capability::CAP_NET_CONNECT)?;
+        let (mut addr, root) = canon_addr(request, &sandbox, &addr, Capability::CAP_NET_CONNECT)?;
         match addr_family(&addr) {
-            PF_UNIX | PF_INET | PF_INET6 => {
+            PF_UNIX => {
+                // Check for access.
+                sandbox_addr(
+                    request,
+                    &sandbox,
+                    &addr,
+                    &root,
+                    0x10,
+                    Capability::CAP_NET_CONNECT,
+                )?;
+            }
+            PF_INET => {
+                // Turn 0.0.0.0 into 127.0.0.1 unless trace/allow_unsafe_any_addr:1.
+                // Turn all addresses to 127.0.0.1 if trace/force_local_net:1.
+                if !any_addr {
+                    make_loaddr(&mut addr, local_net);
+                }
+                // Check for access.
+                sandbox_addr(
+                    request,
+                    &sandbox,
+                    &addr,
+                    &root,
+                    0x10,
+                    Capability::CAP_NET_CONNECT,
+                )?;
+            }
+            PF_INET6 => {
+                // Turn :: into :1 unless trace/allow_unsafe_any_addr:1.
+                // Turn all addresses to ::1 if trace/force_local_net:1.
+                if !any_addr {
+                    make_lo6addr(&mut addr, local_net);
+                }
                 // Check for access.
                 sandbox_addr(
                     request,
@@ -124,7 +159,7 @@ pub(crate) fn handle_sendmsg(
         #[expect(clippy::useless_conversion)]
         let cmsg_len = usize::try_from(msg.msg_controllen)
             .or(Err(Errno::EINVAL))?
-            .min(UIO_MAXLEN); // SAFETY: Cap at UIO_MAXLEN.
+            .min(*MAX_RW_COUNT); // SAFETY: Cap at MAX_RW_COUNT.
         let cmsg_buf = request.read_vec(msg.msg_control as u64, cmsg_len)?;
         Some(parse_control_messages(request, &cmsg_buf)?)
     } else {
@@ -176,7 +211,7 @@ pub(crate) fn handle_sendmsg(
         for fds in control_fds {
             // SAFETY: Check safe file types for SCM_RIGHTS.
             for fd in fds {
-                request.check_scm_rights(fd, flags, 0x10, verbose)?;
+                request.check_scm_rights(fd, flags, 0x10, log_scmp)?;
             }
 
             // SAFETY: OwnedFd is repr(transparent) over RawFd, so
@@ -277,7 +312,7 @@ pub(crate) fn handle_sendmsg(
             // Validate length to avoid large allocations.
             #[expect(clippy::useless_conversion)]
             let iov_len = usize::try_from(iov.iov_len).or(Err(Errno::EMSGSIZE))?;
-            if iov_len > UIO_MAXLEN {
+            if iov_len > *MAX_RW_COUNT {
                 // Invalid iovec count: Too large.
                 return Err(Errno::EMSGSIZE);
             }
@@ -426,12 +461,47 @@ pub(crate) fn handle_sendmmsg(
         let addr_remote = msg.msg_name;
         let addr_len = msg.msg_namelen;
         let sandbox = request.get_sandbox();
-        let verbose = sandbox.verbose;
+        let any_addr = sandbox.flags.allow_unsafe_any_addr();
+        let local_net = sandbox.flags.force_local_net();
+        let log_scmp = sandbox.log_scmp();
         let addr_root = if !addr_remote.is_null() && addr_len > 0 {
             let addr = get_addr(request, addr_remote as u64, addr_len)?;
-            let (addr, root) = canon_addr(request, &sandbox, &addr, Capability::CAP_NET_CONNECT)?;
+            let (mut addr, root) =
+                canon_addr(request, &sandbox, &addr, Capability::CAP_NET_CONNECT)?;
             match addr_family(&addr) {
-                PF_UNIX | PF_INET | PF_INET6 => {
+                PF_UNIX => {
+                    // Check for access.
+                    sandbox_addr(
+                        request,
+                        &sandbox,
+                        &addr,
+                        &root,
+                        0x14,
+                        Capability::CAP_NET_CONNECT,
+                    )?;
+                }
+                PF_INET => {
+                    // Turn 0.0.0.0 into 127.0.0.1 unless trace/allow_unsafe_any_addr:1.
+                    // Turn all addresses to 127.0.0.1 if trace/force_local_net:1.
+                    if !any_addr {
+                        make_loaddr(&mut addr, local_net);
+                    }
+                    // Check for access.
+                    sandbox_addr(
+                        request,
+                        &sandbox,
+                        &addr,
+                        &root,
+                        0x14,
+                        Capability::CAP_NET_CONNECT,
+                    )?;
+                }
+                PF_INET6 => {
+                    // Turn :: into :1 unless trace/allow_unsafe_any_addr:1.
+                    // Turn all addresses to ::1 if trace/force_local_net:1.
+                    if !any_addr {
+                        make_lo6addr(&mut addr, local_net);
+                    }
                     // Check for access.
                     sandbox_addr(
                         request,
@@ -465,7 +535,7 @@ pub(crate) fn handle_sendmmsg(
             #[expect(clippy::useless_conversion)]
             let cmsg_len = usize::try_from(msg.msg_controllen)
                 .or(Err(Errno::EINVAL))?
-                .min(UIO_MAXLEN); // SAFETY: Cap at UIO_MAXLEN.
+                .min(*MAX_RW_COUNT); // SAFETY: Cap at MAX_RW_COUNT.
             let cmsg_buf = request.read_vec(msg.msg_control as u64, cmsg_len)?;
             Some(parse_control_messages(request, &cmsg_buf)?)
         } else {
@@ -521,7 +591,7 @@ pub(crate) fn handle_sendmmsg(
                 for fds in control_fds {
                     // SAFETY: Check safe file types for SCM_RIGHTS.
                     for fd in fds {
-                        request.check_scm_rights(fd, flags, 0x14, verbose)?;
+                        request.check_scm_rights(fd, flags, 0x14, log_scmp)?;
                     }
 
                     // SAFETY: OwnedFd is repr(transparent) over RawFd, so
@@ -626,7 +696,7 @@ pub(crate) fn handle_sendmmsg(
                 // Validate length to avoid large allocations.
                 #[expect(clippy::useless_conversion)]
                 let iov_len = usize::try_from(iov.iov_len).or(Err(Errno::EMSGSIZE))?;
-                if iov_len > UIO_MAXLEN {
+                if iov_len > *MAX_RW_COUNT {
                     // Invalid iovec count: Too large.
                     return Err(Errno::EMSGSIZE);
                 }
