@@ -14,8 +14,8 @@ use nix::errno::Errno;
 use crate::{
     compat::RenameFlags,
     cookie::safe_renameat2,
-    fs::FsFlags,
     kernel::syscall_path_handler,
+    lookup::FsFlags,
     req::{PathArgs, SysArg, UNotifyEventRequest},
 };
 
