@@ -10,9 +10,10 @@ use std::os::fd::AsRawFd;
 
 use libc::c_int;
 use libseccomp::{ScmpArch, ScmpNotifResp};
-use nix::{errno::Errno, fcntl::FallocateFlags};
+use nix::errno::Errno;
 
 use crate::{
+    compat::FallocateFlags,
     confine::{scmp_arch_bits, scmp_big_endian},
     cookie::{safe_ftruncate, safe_ftruncate64, safe_truncate, safe_truncate64},
     fs::{fallocate64, is_valid_fd},
@@ -395,26 +396,19 @@ pub(crate) fn sys_fallocate(request: UNotifyEventRequest) -> ScmpNotifResp {
 }
 
 fn to_fallocate_flags(arg: u64) -> Result<FallocateFlags, Errno> {
-    // TODO: Remove FALLOC_FL_WRITE_ZEROES logic once nix supports the flag.
     // FALLOC_FL_MODE_MASK: FL_ALLOCATE_RANGE is encoded as 0, so it doesn't affect the mask.
-    const FALLOC_FL_WRITE_ZEROES: FallocateFlags = FallocateFlags::from_bits_retain(0x80);
     const FALLOC_FL_MODE_MASK: c_int = FallocateFlags::FALLOC_FL_PUNCH_HOLE.bits()
         | FallocateFlags::FALLOC_FL_COLLAPSE_RANGE.bits()
         | FallocateFlags::FALLOC_FL_ZERO_RANGE.bits()
         | FallocateFlags::FALLOC_FL_INSERT_RANGE.bits()
         | FallocateFlags::FALLOC_FL_UNSHARE_RANGE.bits()
-        | FALLOC_FL_WRITE_ZEROES.bits();
+        | FallocateFlags::FALLOC_FL_WRITE_ZEROES.bits();
 
     let arg: c_int = arg.try_into().or(Err(Errno::EINVAL))?;
-    let all: c_int = FallocateFlags::all().bits() | FALLOC_FL_WRITE_ZEROES.bits();
-    if (arg & !all) != 0 {
+    if (arg & !FallocateFlags::all().bits()) != 0 {
         return Err(Errno::EINVAL);
     }
-
-    let mut flags = FallocateFlags::from_bits_truncate(arg);
-    if (arg & FALLOC_FL_WRITE_ZEROES.bits()) != 0 {
-        flags |= FALLOC_FL_WRITE_ZEROES;
-    }
+    let flags = FallocateFlags::from_bits_truncate(arg);
 
     // See Linux' vfs_fallocate() in fs/open.c for the validations.
     if (arg & !(FALLOC_FL_MODE_MASK | FallocateFlags::FALLOC_FL_KEEP_SIZE.bits())) != 0 {
@@ -432,7 +426,7 @@ fn to_fallocate_flags(arg: u64) -> Result<FallocateFlags, Errno> {
         }
         x if x == FallocateFlags::FALLOC_FL_COLLAPSE_RANGE.bits()
             || x == FallocateFlags::FALLOC_FL_INSERT_RANGE.bits()
-            || x == FALLOC_FL_WRITE_ZEROES.bits() =>
+            || x == FallocateFlags::FALLOC_FL_WRITE_ZEROES.bits() =>
         {
             if (arg & FallocateFlags::FALLOC_FL_KEEP_SIZE.bits()) != 0 {
                 return Err(Errno::EOPNOTSUPP);
