@@ -9,14 +9,17 @@ use std::{
     },
 };
 
+use bitflags::bitflags;
 use libc::close;
 
 #[cfg(test)]
 use crate::landlock::*;
 use crate::landlock::{
-    access::PrivateAccess, compat::private::OptionCompatLevelMut, uapi, Access, AccessFs,
-    AccessNet, AddRuleError, AddRulesError, CompatLevel, CompatState, Compatibility, Compatible,
-    CreateRulesetError, RestrictSelfError, RulesetError, Scope, TryCompat,
+    access::PrivateAccess,
+    compat::{private::OptionCompatLevelMut, LandlockStatus, ABI},
+    uapi, Access, AccessFs, AccessNet, AddRuleError, AddRulesError, CompatLevel, CompatState,
+    Compatibility, Compatible, CreateRulesetError, HandleAccessError, HandleAccessesError,
+    RestrictSelfError, RulesetError, Scope, TryCompat,
 };
 
 // Public interface without methods and which is impossible to implement outside this crate.
@@ -77,6 +80,8 @@ pub struct RestrictionStatus {
     pub ruleset: RulesetStatus,
     /// Status of `prctl(2)`'s `PR_SET_NO_NEW_PRIVS` enforcement.
     pub no_new_privs: bool,
+    /// Status of Landlock for the running kernel.
+    pub landlock: LandlockStatus,
 }
 
 fn prctl_set_no_new_privs() -> Result<(), Error> {
@@ -191,7 +196,7 @@ pub struct Ruleset {
 impl From<Compatibility> for Ruleset {
     fn from(compat: Compatibility) -> Self {
         Ruleset {
-            // Non-working default handled FS accesses to force users to set them explicitely.
+            // Non-working default handled FS accesses to force users to set them explicitly.
             requested_handled_fs: Default::default(),
             requested_handled_net: Default::default(),
             requested_scoped: Default::default(),
@@ -244,7 +249,6 @@ impl Default for Ruleset {
 }
 
 impl Ruleset {
-    #[allow(clippy::new_without_default)]
     #[deprecated(note = "Use Ruleset::default() instead")]
     pub fn new() -> Self {
         Ruleset::default()
@@ -373,7 +377,19 @@ pub trait RulesetAttr: Sized + AsMut<Ruleset> + Compatible {
     where
         T: Into<Scope>,
     {
-        Scope::ruleset_handle_access(self.as_mut(), scope.into())?;
+        let scope = scope.into();
+        let ruleset = self.as_mut();
+        ruleset.requested_scoped |= scope;
+        if let Some(a) = scope
+            .try_compat(
+                ruleset.compat.abi(),
+                ruleset.compat.level,
+                &mut ruleset.compat.state,
+            )
+            .map_err(|err| HandleAccessesError::Scope(HandleAccessError::Compat(err)))?
+        {
+            ruleset.actual_scoped |= a;
+        }
         Ok(self)
     }
 }
@@ -454,6 +470,17 @@ fn ruleset_created_handle_access_net_tcp() {
     assert_eq!(ruleset.requested_handled_net, access);
     assert_eq!(ruleset.actual_handled_net, access);
 
+    // Tests composition (binary OR) of handled accesses.
+    let ruleset = Ruleset::from(ABI::V4)
+        .handle_access(AccessNet::BindTcp)
+        .unwrap()
+        .handle_access(AccessNet::ConnectTcp)
+        .unwrap()
+        .handle_access(AccessNet::BindTcp)
+        .unwrap();
+    assert_eq!(ruleset.requested_handled_net, access);
+    assert_eq!(ruleset.actual_handled_net, access);
+
     // Tests that only the required handled accesses are reported as incompatible:
     // access should not contains AccessNet::BindTcp.
     assert!(matches!(Ruleset::from(ABI::Unsupported)
@@ -701,6 +728,114 @@ pub trait RulesetCreatedAttr: Sized + AsMut<RulesetCreated> + Compatible {
     }
 }
 
+bitflags! {
+    /// By default, denied accesses originating from programs that sandbox themselves
+    /// are logged via the audit subsystem. Such events typically indicate unexpected
+    /// behavior, such as bugs or exploitation attempts. However, to avoid excessive
+    /// logging, access requests denied by a domain not created by the originating
+    /// program are not logged by default. The rationale is that programs should know
+    /// their own behavior, but not necessarily the behavior of other programs.  This
+    /// default configuration is suitable for most programs that sandbox themselves.
+    /// For specific use cases, the following flags allow programs to modify this
+    /// default logging behavior.
+    ///
+    /// The `LANDLOCK_RESTRICT_SELF_LOG_SAME_EXEC_OFF` and
+    /// `LANDLOCK_RESTRICT_SELF_LOG_NEW_EXEC_ON` flags apply to the newly created
+    /// Landlock domain.
+    ///
+    /// `LANDLOCK_RESTRICT_SELF_LOG_SAME_EXEC_OFF`
+    ///     Disables logging of denied accesses originating from the thread creating
+    ///     the Landlock domain, as well as its children, as long as they continue
+    ///     running the same executable code (i.e., without an intervening
+    ///     `execve(2)` call). This is intended for programs that execute
+    ///     unknown code without invoking `execve(2)`, such as script
+    ///     interpreters. Programs that only sandbox themselves should not set this
+    ///     flag, so users can be notified of unauthorized access attempts via system
+    ///     logs.
+    ///
+    /// `LANDLOCK_RESTRICT_SELF_LOG_NEW_EXEC_ON`
+    ///     Enables logging of denied accesses after an `execve(2)` call,
+    ///     providing visibility into unauthorized access attempts by newly executed
+    ///     programs within the created Landlock domain. This flag is recommended
+    ///     only when all potential executables in the domain are expected to comply
+    ///     with the access restrictions, as excessive audit log entries could make
+    ///     it more difficult to identify critical events.
+    ///
+    /// `LANDLOCK_RESTRICT_SELF_LOG_SUBDOMAINS_OFF`
+    ///     Disables logging of denied accesses originating from nested Landlock
+    ///     domains created by the caller or its descendants. This flag should be set
+    ///     according to runtime configuration, not hardcoded, to avoid suppressing
+    ///     important security events. It is useful for container runtimes or
+    ///     sandboxing tools that may launch programs which themselves create
+    ///     Landlock domains and could otherwise generate excessive logs. Unlike
+    ///     `LANDLOCK_RESTRICT_SELF_LOG_SAME_EXEC_OFF`, this flag only affects
+    ///     future nested domains, not the one being created. It can also be used
+    ///     with a `ruleset_fd` value of -1 to mute subdomain logs without creating a
+    ///     domain.
+    #[derive(Clone, Copy, Debug, Default, Eq, PartialEq, Hash)]
+    pub struct RestrictSelfFlags: u32 {
+        /// `LANDLOCK_RESTRICT_SELF_LOG_SAME_EXEC_OFF`
+        const LOG_SAME_EXEC_OFF  = uapi::LANDLOCK_RESTRICT_SELF_LOG_SAME_EXEC_OFF;
+        /// `LANDLOCK_RESTRICT_SELF_LOG_NEW_EXEC_ON`
+        const LOG_NEW_EXEC_ON    = uapi::LANDLOCK_RESTRICT_SELF_LOG_NEW_EXEC_ON;
+        /// `LANDLOCK_RESTRICT_SELF_LOG_SUBDOMAINS_OFF`
+        const LOG_SUBDOMAINS_OFF = uapi::LANDLOCK_RESTRICT_SELF_LOG_SUBDOMAINS_OFF;
+
+        /// Flags supported by landlock(7) ABI 7.
+        const MASK_V7 =
+            Self::LOG_SAME_EXEC_OFF.bits() |
+            Self::LOG_NEW_EXEC_ON.bits() |
+            Self::LOG_SUBDOMAINS_OFF.bits();
+    }
+}
+
+impl RestrictSelfFlags {
+    /// Returns the mask of flags supported by the given ABI.
+    pub const fn supported(abi: ABI) -> Self {
+        match abi {
+            ABI::Unsupported | ABI::V1 | ABI::V2 | ABI::V3 | ABI::V4 | ABI::V5 | ABI::V6 => {
+                Self::empty()
+            }
+            ABI::V7 => Self::MASK_V7,
+        }
+    }
+
+    /// Returns `self` with unsupported bits stripped for the given ABI.
+    #[inline]
+    pub fn retain_supported(self, abi: ABI) -> Self {
+        self & Self::supported(abi)
+    }
+
+    /// Returns the subset of `self` that is *not* supported on the given ABI.
+    #[inline]
+    pub fn unsupported(self, abi: ABI) -> Self {
+        self & !Self::supported(abi)
+    }
+}
+
+impl std::fmt::Display for RestrictSelfFlags {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        const FLAGS: &[(RestrictSelfFlags, &str)] = &[
+            (RestrictSelfFlags::LOG_SAME_EXEC_OFF, "log_same_exec_off"),
+            (RestrictSelfFlags::LOG_NEW_EXEC_ON, "log_new_exec_on"),
+            (RestrictSelfFlags::LOG_SUBDOMAINS_OFF, "log_subdomains_off"),
+        ];
+
+        let mut first = true;
+        for (flag, name) in FLAGS {
+            if self.contains(*flag) {
+                if !first {
+                    f.write_str(",")?;
+                }
+                f.write_str(name)?;
+                first = false;
+            }
+        }
+
+        Ok(())
+    }
+}
+
 /// Ruleset created with [`Ruleset::create()`].
 #[cfg_attr(test, derive(Debug))]
 pub struct RulesetCreated {
@@ -733,7 +868,10 @@ impl RulesetCreated {
     /// according to the ruleset configuration.
     ///
     /// On error, returns a wrapped [`RestrictSelfError`].
-    pub fn restrict_self(mut self) -> Result<RestrictionStatus, RulesetError> {
+    pub fn restrict_self(
+        mut self,
+        flags: RestrictSelfFlags,
+    ) -> Result<RestrictionStatus, RulesetError> {
         let mut body = || -> Result<RestrictionStatus, RestrictSelfError> {
             // Enforce no_new_privs even if something failed with SoftRequirement. The rationale is
             // that no_new_privs should not be an issue on its own if it is not explicitly
@@ -779,6 +917,7 @@ impl RulesetCreated {
             match self.compat.state {
                 CompatState::Init | CompatState::No | CompatState::Dummy => Ok(RestrictionStatus {
                     ruleset: self.compat.state.into(),
+                    landlock: self.compat.status(),
                     no_new_privs: enforced_nnp,
                 }),
                 CompatState::Full | CompatState::Partial => {
@@ -786,11 +925,20 @@ impl RulesetCreated {
                     assert!(self.fd.is_some());
                     // Does not consume ruleset FD, which will be automatically closed after this block.
                     let fd = self.fd.as_ref().map(|f| f.as_raw_fd()).unwrap_or(-1);
-                    match unsafe { uapi::landlock_restrict_self(fd, 0) } {
+
+                    // Determine flags supported by the current ABI.
+                    //
+                    // Unsupported flags don't throw an error when
+                    // the compatibility mode is hard-requirement but
+                    // this may change in the future with new flags.
+                    let flags = flags.retain_supported(self.compat.abi());
+
+                    match unsafe { uapi::landlock_restrict_self(fd, flags.bits()) } {
                         0 => {
                             self.compat.update(CompatState::Full);
                             Ok(RestrictionStatus {
                                 ruleset: self.compat.state.into(),
+                                landlock: self.compat.status(),
                                 no_new_privs: enforced_nnp,
                             })
                         }
@@ -884,10 +1032,11 @@ fn ruleset_created_attr() {
                 AccessFs::Execute,
             ))
             .unwrap()
-            .restrict_self()
+            .restrict_self(RestrictSelfFlags::empty())
             .unwrap(),
         RestrictionStatus {
             ruleset: RulesetStatus::NotEnforced,
+            landlock: LandlockStatus::NotImplemented,
             no_new_privs: true,
         }
     );
@@ -957,10 +1106,11 @@ fn ruleset_unsupported() {
             .unwrap()
             .create()
             .unwrap()
-            .restrict_self()
+            .restrict_self(RestrictSelfFlags::empty())
             .unwrap(),
         RestrictionStatus {
             ruleset: RulesetStatus::NotEnforced,
+            landlock: LandlockStatus::NotImplemented,
             // With BestEffort, no_new_privs is still enabled.
             no_new_privs: true,
         }
@@ -974,10 +1124,11 @@ fn ruleset_unsupported() {
             .unwrap()
             .create()
             .unwrap()
-            .restrict_self()
+            .restrict_self(RestrictSelfFlags::empty())
             .unwrap(),
         RestrictionStatus {
             ruleset: RulesetStatus::NotEnforced,
+            landlock: LandlockStatus::NotImplemented,
             // With SoftRequirement, no_new_privs is still enabled.
             no_new_privs: true,
         }
@@ -1011,10 +1162,11 @@ fn ruleset_unsupported() {
             .unwrap()
             // SoftRequirement for RulesetCreated without any rule.
             .set_compatibility(CompatLevel::SoftRequirement)
-            .restrict_self()
+            .restrict_self(RestrictSelfFlags::empty())
             .unwrap(),
         RestrictionStatus {
             ruleset: RulesetStatus::NotEnforced,
+            landlock: LandlockStatus::NotImplemented,
             // With SoftRequirement, no_new_privs is untouched if there is no error (e.g. no rule).
             no_new_privs: true,
         }
@@ -1032,10 +1184,11 @@ fn ruleset_unsupported() {
                 .set_compatibility(CompatLevel::SoftRequirement)
                 .add_rule(PathBeneath::new(PathFd::new("/").unwrap(), AccessFs::Refer))
                 .unwrap()
-                .restrict_self()
+                .restrict_self(RestrictSelfFlags::empty())
                 .unwrap(),
             RestrictionStatus {
                 ruleset: RulesetStatus::NotEnforced,
+                landlock: LandlockStatus::Available(ABI::V1, None),
                 // With SoftRequirement, no_new_privs is still enabled, even if there is an error
                 // (e.g. unsupported access right).
                 no_new_privs: true,
@@ -1050,10 +1203,11 @@ fn ruleset_unsupported() {
             .create()
             .unwrap()
             .set_no_new_privs(false)
-            .restrict_self()
+            .restrict_self(RestrictSelfFlags::empty())
             .unwrap(),
         RestrictionStatus {
             ruleset: RulesetStatus::NotEnforced,
+            landlock: LandlockStatus::NotImplemented,
             no_new_privs: false,
         }
     );
@@ -1156,10 +1310,11 @@ fn ignore_abi_v2_with_abi_v1() {
                 make_bitflags!(AccessFs::{ReadFile | ReadDir})
             ))
             .unwrap()
-            .restrict_self()
+            .restrict_self(RestrictSelfFlags::empty())
             .unwrap(),
         RestrictionStatus {
             ruleset: RulesetStatus::NotEnforced,
+            landlock: LandlockStatus::Available(ABI::V1, None),
             no_new_privs: true,
         }
     );
