@@ -14,11 +14,11 @@ use std::sync::LazyLock;
 use memchr::arch::all::is_equal;
 use nix::{
     errno::Errno,
-    fcntl::{open, OFlag},
-    sys::stat::Mode,
+    fcntl::{OFlag, AT_FDCWD},
 };
 
 use crate::{
+    compat::{openat2, OpenHow, ResolveFlag},
     hash::{SydHashMap, SydHashSet, SydIndexMap},
     landlock::{
         Access, AccessFs, AccessNet, CompatLevel, Compatible, CreateRulesetError, Errata, NetPort,
@@ -29,6 +29,7 @@ use crate::{
         str2u32, LandlockCmd, LandlockOp, LandlockRule, PathSet, PortRange, PortSet,
     },
     path::{XPath, XPathBuf},
+    retry::retry_on_eintr,
     sandbox::Sandbox,
 };
 
@@ -764,7 +765,22 @@ where
 
     paths.into_iter().filter_map(move |p| {
         let p = p.as_ref();
-        match open(p, OFlag::O_PATH | OFlag::O_CLOEXEC, Mode::empty()) {
+        if p.has_parent_dot() {
+            crate::error!("ctx": "init", "op": "landlock_create_ruleset",
+                "path": p, "access": access,
+                "cmp": compat_level, "err": Errno::EACCES as i32,
+                "msg": format!("open path `{p}' for Landlock failed due to dotdot"),
+                "tip": "avoid using `..' components in Landlock rules");
+            return Some(Err(RulesetError::CreateRuleset(
+                CreateRulesetError::CreateRulesetCall {
+                    source: Errno::EACCES.into(),
+                },
+            )));
+        }
+        let how = OpenHow::new()
+            .flags(OFlag::O_PATH | OFlag::O_CLOEXEC)
+            .resolve(ResolveFlag::RESOLVE_NO_MAGICLINKS);
+        match retry_on_eintr(|| openat2(AT_FDCWD, p, how)) {
             Ok(fd) => Some(Ok(PathBeneath::new(PathFd { fd }, access))),
             Err(errno @ Errno::ENOENT) if level == CompatLevel::BestEffort => {
                 crate::info!("ctx": "init", "op": "landlock_create_ruleset",
@@ -774,11 +790,18 @@ where
                 None
             }
             Err(errno) => {
+                let tip = if errno == Errno::ENOENT {
+                    "set `default/lock:warn' to ignore file-not-found errors for Landlock"
+                } else if errno == Errno::ELOOP {
+                    "avoid using magiclinks in Landlock rules"
+                } else {
+                    "avoid using inaccessible paths in Landlock rules"
+                };
                 crate::error!("ctx": "init", "op": "landlock_create_ruleset",
                     "path": p, "access": access,
                     "cmp": compat_level, "err": errno as i32,
                     "msg": format!("open path `{p}' for Landlock failed: {errno}"),
-                    "tip": "set `default/lock:warn' to ignore file-not-found errors for Landlock");
+                    "tip": tip);
                 Some(Err(RulesetError::CreateRuleset(
                     CreateRulesetError::CreateRulesetCall {
                         source: errno.into(),
