@@ -20,6 +20,7 @@ use std::{
 
 use btoi::btoi;
 use data_encoding::HEXLOWER;
+use libseccomp::ScmpArch;
 use nix::{
     errno::Errno,
     unistd::{write, Pid, Uid},
@@ -30,6 +31,7 @@ use crate::{
     config::*,
     err::SydResult,
     fs::is_active_fd,
+    ioctl::{Ioctl, IoctlMap, IoctlName},
     ofd::{lock_fd, unlock_fd},
     proc::{proc_cmdline, proc_comm, proc_cwd, proc_tty},
     retry::retry_on_eintr,
@@ -511,6 +513,12 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
         msg.insert("uid".to_string(), Uid::current().as_raw().into());
     }
 
+    // Add "id" = sandbox ID.
+    #[expect(clippy::disallowed_methods)]
+    if let Ok(id) = std::env::var(ENV_ID) {
+        msg.insert("id".to_string(), id.into());
+    }
+
     // Add "syd" = current thread ID.
     let syd = nix::unistd::gettid().as_raw().into();
     msg.insert("syd".to_string(), Value::Number(syd));
@@ -686,6 +694,12 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
         msg.insert("uid".to_string(), Uid::current().as_raw().into());
     }
 
+    // Add "id" = sandbox ID.
+    #[expect(clippy::disallowed_methods)]
+    if let Ok(id) = std::env::var(ENV_ID) {
+        msg.insert("id".to_string(), id.into());
+    }
+
     // Add "syd" = current thread ID.
     let syd = nix::unistd::gettid().as_raw().into();
     msg.insert("syd".to_string(), Value::Number(syd));
@@ -792,6 +806,31 @@ pub struct Tm {
     tm_sec: i32,  // 0..60 (leap second tolerated like musl)
 }
 
+/// Helper to validly allocate and return ioctl names for logging.
+// Conversion is necessary on 32-bit.
+#[expect(clippy::useless_conversion)]
+pub fn get_ioctl_log(
+    val: Ioctl,
+    arch: ScmpArch,
+    resolve: bool,
+) -> Result<Option<Vec<IoctlName>>, Errno> {
+    if !resolve {
+        let mut vec = Vec::new();
+        vec.try_reserve(1).or(Err(Errno::ENOMEM))?;
+        vec.push(IoctlName::Val(val.into()));
+        return Ok(Some(vec));
+    }
+    match IoctlMap::new(None, true).get_log(val, arch)? {
+        Some(names) => Ok(Some(names)),
+        None => {
+            let mut vec = Vec::new();
+            vec.try_reserve(1).or(Err(Errno::ENOMEM))?;
+            vec.push(IoctlName::Val(val.into()));
+            Ok(Some(vec))
+        }
+    }
+}
+
 impl Tm {
     /// Get the year of the date.
     pub fn year(&self) -> i32 {
