@@ -13,18 +13,19 @@ use std::{
     os::fd::{AsFd, AsRawFd, BorrowedFd, OwnedFd, RawFd},
 };
 
-use libc::{c_int, dup3, GRND_RANDOM};
+use libc::{c_int, GRND_RANDOM};
 use memchr::memchr;
 use nix::{
     errno::Errno,
-    fcntl::{OFlag, ResolveFlag},
+    fcntl::OFlag,
     sys::resource::{getrlimit, Resource},
-    unistd::{close, UnlinkatFlags},
+    unistd::UnlinkatFlags,
 };
 
 use crate::{
+    compat::{dup3, ResolveFlag},
     cookie::safe_unlinkat,
-    fs::is_active_fd,
+    fs::{close, is_active_fd},
     log::{now, Tm},
     lookup::safe_open,
     path::{XPathBuf, PATH_MAX},
@@ -273,10 +274,7 @@ pub fn duprand(oldfd: RawFd, mut flags: OFlag) -> Result<RawFd, Errno> {
             continue;
         }
 
-        match retry_on_eintr(|| {
-            // SAFETY: In libc we trust.
-            Errno::result(unsafe { dup3(oldfd, fd_rand.as_raw_fd(), flags.bits()) })
-        }) {
+        match retry_on_eintr(|| dup3(oldfd, fd_rand.as_raw_fd(), flags.bits())) {
             Ok(_) => {
                 if close_old {
                     let _ = close(oldfd);
