@@ -76,7 +76,6 @@ use nix::{
     pty::{grantpt, unlockpt},
     sched::{unshare, CloneFlags},
     sys::{
-        prctl::{set_name, set_pdeathsig},
         signal::Signal,
         socket::{bind, socket, AddressFamily, SockFlag, SockType, SockaddrIn, SockaddrIn6},
         stat::{umask, Mode},
@@ -88,7 +87,7 @@ use nix::{
 use sendfd::SendWithFd;
 use syd::{
     caps,
-    compat::{waitid, WaitStatus},
+    compat::{set_name, set_pdeathsig, waitid, WaitStatus},
     config::*,
     confine::{confine_scmp_madvise, ExportMode},
     err::err2no,
@@ -116,6 +115,7 @@ use syd::{
 #[cfg(all(
     not(coverage),
     not(feature = "prof"),
+    not(target_os = "android"),
     target_page_size_4k,
     target_pointer_width = "64"
 ))]
@@ -418,7 +418,6 @@ syd::main! {
                         .map(|arg| ExportMode::from_str(&arg))??,
                 );
             }
-            Short('x') => sandbox.parse_profile("trace")?,
             Short('m') => {
                 let cmd = parser.value().map(XPathBuf::from)?;
                 if sandbox.locked() {
@@ -428,6 +427,19 @@ syd::main! {
                     sandbox.config(&cmd.to_string())?;
                 }
             }
+            Short('t') => {
+                let tmout = parser.value()
+                    .ok()
+                    .and_then(|ostr| ostr.into_string().ok())
+                    .ok_or(Errno::EINVAL)?;
+                if sandbox.locked() {
+                    eprintln!("Failed to set sandbox timeout: sandbox locked!");
+                    return Err(Errno::EBUSY.into());
+                } else {
+                    sandbox.config(&format!("timeout:{tmout}"))?;
+                }
+            }
+            Short('x') => sandbox.parse_profile("trace")?,
             Short('f') => {
                 // Login shell compatibility:
                 // Parse user profile as necessary.
@@ -526,6 +538,8 @@ syd::main! {
         // Note, we do not intervene with sandbox policy here, and let
         // the user configure it through other means. This way the user
         // can dump seccomp filters for different set of options.
+    } else {
+        env::remove_var(ENV_DUMP_SCMP);
     }
 
     if user_parse && !user_done && !sandbox.locked() {
