@@ -7,7 +7,7 @@
 //
 // SPDX-License-Identifier: GPL-3.0
 
-use std::{process::ExitCode, str::FromStr, time::Duration};
+use std::{os::unix::ffi::OsStrExt, process::ExitCode, str::FromStr, time::Duration};
 
 use libc::c_int;
 use libseccomp::{ScmpArch, ScmpSyscall};
@@ -26,6 +26,7 @@ use syd::{
     confine::print_seccomp_architectures,
     fs::unix_inodes,
     ioctl::{Ioctl, IoctlMap},
+    parsers::sandbox::str2u64,
     proc::proc_unix_inodes,
     wildmatch::inamematch,
 };
@@ -34,6 +35,7 @@ use syd::{
 #[cfg(all(
     not(coverage),
     not(feature = "prof"),
+    not(target_os = "android"),
     target_page_size_4k,
     target_pointer_width = "64"
 ))]
@@ -207,10 +209,10 @@ syd::main! {
             }
         };
     } else if opt_ioctl {
-        let ioctl = IoctlMap::new(false);
-        return match sysarg.parse::<Ioctl>() {
+        let ioctl = IoctlMap::new(Some(opt_arch), false);
+        return match str2u64(sysarg.as_bytes()).map(|arg| arg as Ioctl) {
             Ok(num) => {
-                if let Some(names) = ioctl.get_names(num, opt_arch) {
+                if let Ok(Some(names)) = ioctl.get_names(num, opt_arch) {
                     for name in names {
                         println!("{name}\t{num}");
                     }
@@ -221,15 +223,13 @@ syd::main! {
             }
             Err(_) => {
                 let glob = sysarg.to_str().ok_or(Errno::EINVAL)?;
-                let list = ioctl.get_map(opt_arch).ok_or(Errno::EINVAL)?;
+                let iter = ioctl.iter(opt_arch).ok_or(Errno::EINVAL)?;
 
                 let mut ok = false;
-                for (num, names) in list {
-                    for name in names {
-                        if inamematch(glob, name) {
-                            println!("{name}\t{num}");
-                            ok = true;
-                        }
+                for (name, num) in iter {
+                    if inamematch(glob, name) {
+                        println!("{name}\t{num}");
+                        ok = true;
                     }
                 }
 
