@@ -29,7 +29,7 @@ use nix::{
     errno::Errno,
     fcntl::OFlag,
     sched::{unshare, CloneFlags},
-    unistd::{close, Gid, Uid},
+    unistd::{Gid, Uid},
 };
 
 #[cfg(target_arch = "x86")]
@@ -38,33 +38,37 @@ use crate::cookie::FTRUNCATE64_COOKIE_ARG3;
 use crate::cookie::TRUNCATE64_COOKIE_ARG3;
 use crate::{
     alert,
+    compat::{seccomp_notif, seccomp_notif_resp},
     config::*,
     confine::{
-        confine_scmp_clone, confine_scmp_clone3, confine_scmp_madvise, extend_ioctl,
+        confine_scmp, confine_scmp_clone, confine_scmp_clone3, confine_scmp_madvise, extend_ioctl,
         scmp_add_setid_rules, scmp_arch_raw, ScmpNotifReq, SydArch, Sydcall, EIDRM, EOWNERDEAD,
         X32_SYSCALL_BIT,
     },
     cookie::{
         ACCEPT4_COOKIE_ARG4, ACCEPT4_COOKIE_ARG5, BIND_COOKIE_ARG3, BIND_COOKIE_ARG4,
         BIND_COOKIE_ARG5, CONNECT_COOKIE_ARG3, CONNECT_COOKIE_ARG4, CONNECT_COOKIE_ARG5,
-        FTRUNCATE64_COOKIE_ARG4, FTRUNCATE64_COOKIE_ARG5, FTRUNCATE_COOKIE_ARG2,
-        FTRUNCATE_COOKIE_ARG3, FTRUNCATE_COOKIE_ARG4, FTRUNCATE_COOKIE_ARG5, LINKAT_COOKIE_ARG5,
-        MEMFD_CREATE_COOKIE_ARG2, MEMFD_CREATE_COOKIE_ARG3, MEMFD_CREATE_COOKIE_ARG4,
-        MEMFD_CREATE_COOKIE_ARG5, OPENAT2_COOKIE_ARG4, OPENAT2_COOKIE_ARG5, PIPE2_COOKIE_ARG2,
-        PIPE2_COOKIE_ARG3, PIPE2_COOKIE_ARG4, PIPE2_COOKIE_ARG5, RENAMEAT2_COOKIE_ARG5,
-        SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG3, SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG4,
-        SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG5, SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG3,
-        SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG4, SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG5,
-        SOCKETPAIR_COOKIE_ARG4, SOCKETPAIR_COOKIE_ARG5, SOCKET_COOKIE_ARG3, SOCKET_COOKIE_ARG4,
-        SOCKET_COOKIE_ARG5, SYS_ACCEPT4, SYS_BIND, SYS_CONNECT, SYS_SOCKET, SYS_SOCKETPAIR,
-        TRUNCATE64_COOKIE_ARG4, TRUNCATE64_COOKIE_ARG5, TRUNCATE_COOKIE_ARG2, TRUNCATE_COOKIE_ARG3,
-        TRUNCATE_COOKIE_ARG4, TRUNCATE_COOKIE_ARG5, UNAME_COOKIE_ARG1, UNAME_COOKIE_ARG2,
-        UNAME_COOKIE_ARG3, UNAME_COOKIE_ARG4, UNAME_COOKIE_ARG5, UNLINKAT_COOKIE_ARG3,
-        UNLINKAT_COOKIE_ARG4, UNLINKAT_COOKIE_ARG5,
+        FCHDIR_COOKIE_ARG1, FCHDIR_COOKIE_ARG2, FCHDIR_COOKIE_ARG3, FCHDIR_COOKIE_ARG4,
+        FCHDIR_COOKIE_ARG5, FTRUNCATE64_COOKIE_ARG4, FTRUNCATE64_COOKIE_ARG5,
+        FTRUNCATE_COOKIE_ARG2, FTRUNCATE_COOKIE_ARG3, FTRUNCATE_COOKIE_ARG4, FTRUNCATE_COOKIE_ARG5,
+        LINKAT_COOKIE_ARG5, MEMFD_CREATE_COOKIE_ARG2, MEMFD_CREATE_COOKIE_ARG3,
+        MEMFD_CREATE_COOKIE_ARG4, MEMFD_CREATE_COOKIE_ARG5, OPENAT2_COOKIE_ARG4,
+        OPENAT2_COOKIE_ARG5, PIPE2_COOKIE_ARG2, PIPE2_COOKIE_ARG3, PIPE2_COOKIE_ARG4,
+        PIPE2_COOKIE_ARG5, RENAMEAT2_COOKIE_ARG5, SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG3,
+        SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG4, SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG5,
+        SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG3, SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG4,
+        SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG5, SOCKETPAIR_COOKIE_ARG4, SOCKETPAIR_COOKIE_ARG5,
+        SOCKET_COOKIE_ARG3, SOCKET_COOKIE_ARG4, SOCKET_COOKIE_ARG5, SYS_ACCEPT4, SYS_BIND,
+        SYS_CONNECT, SYS_SOCKET, SYS_SOCKETPAIR, TRUNCATE64_COOKIE_ARG4, TRUNCATE64_COOKIE_ARG5,
+        TRUNCATE_COOKIE_ARG2, TRUNCATE_COOKIE_ARG3, TRUNCATE_COOKIE_ARG4, TRUNCATE_COOKIE_ARG5,
+        UMASK_COOKIE_ARG1, UMASK_COOKIE_ARG2, UMASK_COOKIE_ARG3, UMASK_COOKIE_ARG4,
+        UMASK_COOKIE_ARG5, UNAME_COOKIE_ARG1, UNAME_COOKIE_ARG2, UNAME_COOKIE_ARG3,
+        UNAME_COOKIE_ARG4, UNAME_COOKIE_ARG5, UNLINKAT_COOKIE_ARG3, UNLINKAT_COOKIE_ARG4,
+        UNLINKAT_COOKIE_ARG5,
     },
     err::{err2no, SydJoinHandle, SydResult},
     fs::{
-        seccomp_notify_respond, AT_EXECVE_CHECK, SECCOMP_IOCTL_NOTIF_ADDFD,
+        close, seccomp_notify_respond, AT_EXECVE_CHECK, SECCOMP_IOCTL_NOTIF_ADDFD,
         SECCOMP_IOCTL_NOTIF_LIST, SECCOMP_IOCTL_NOTIF_SEND,
     },
     hook::HandlerMap,
@@ -74,41 +78,36 @@ use crate::{
     req::UNotifyEventRequest,
     sandbox::{Flags, Sandbox, SandboxGuard},
     timer::AlarmTimer,
-    workers::{aes::AesLock, UnixMap, WorkerCache, WorkerData},
+    workers::{WorkerCache, WorkerData},
 };
 
 #[derive(Clone)]
 pub(crate) struct Worker {
     fd: RawFd,
     flags: Flags,
-    cache: Arc<WorkerCache<'static>>,
+    cache: Arc<WorkerCache>,
     sandbox: Arc<RwLock<Sandbox>>,
     handlers: Arc<HandlerMap>,
     keep_alive: Option<u16>,
     should_exit: Arc<AtomicBool>,
     worker_data: Arc<WorkerData>,
-    crypt_map: Option<AesLock>,
-    unix_map: UnixMap,
 }
 
 impl Worker {
-    #[expect(clippy::too_many_arguments)]
     pub(crate) fn new(
         fd: RawFd,
-        cache: Arc<WorkerCache<'static>>,
+        cache: Arc<WorkerCache>,
         sandbox: Arc<RwLock<Sandbox>>,
         handlers: Arc<HandlerMap>,
         keep_alive: Option<u16>,
         should_exit: Arc<AtomicBool>,
         worker_data: Arc<WorkerData>,
-        crypt_map: Option<AesLock>,
-        unix_map: UnixMap,
     ) -> Self {
         let my_sandbox = SandboxGuard::Read(sandbox.read().unwrap_or_else(|err| err.into_inner()));
         let flags = *my_sandbox.flags;
         drop(my_sandbox); // release the read lock.
 
-        Worker {
+        Self {
             fd,
             flags,
             cache,
@@ -117,8 +116,6 @@ impl Worker {
             keep_alive,
             should_exit,
             worker_data,
-            crypt_map,
-            unix_map,
         }
     }
 
@@ -147,7 +144,7 @@ impl Worker {
                 // emulator threads have to share memory fds with AES
                 // threads.
                 let mut unshare_flags = CloneFlags::CLONE_FS;
-                let is_crypt = self.crypt_map.is_some();
+                let is_crypt = self.cache.crypt_map.is_some();
                 if !is_crypt {
                     unshare_flags.insert(CloneFlags::CLONE_FILES);
                 }
@@ -155,7 +152,7 @@ impl Worker {
                 // SAFETY: We use exit_group(2) here to bail,
                 // because this unsharing is a critical safety feature.
                 if let Err(errno) = unshare(unshare_flags) {
-                    alert!("ctx": "boot", "op": "unshare_emulator_thread",
+                    alert!("ctx": "boot", "op": "unshare_emu_thread",
                         "msg": format!("failed to unshare({unshare_flags:?}): {errno}"),
                         "err": errno as i32);
                     std::process::exit(101);
@@ -167,7 +164,7 @@ impl Worker {
                     match AlarmTimer::from_seconds(keep_alive.into()) {
                         Ok(timer) => Some(timer),
                         Err(errno) => {
-                            alert!("ctx": "boot", "op": "timer_create_for_emulator_thread",
+                            alert!("ctx": "boot", "op": "timer_create_for_emu_thread",
                                 "msg": format!("failed to set up timer: {errno}"),
                                 "err": errno as i32);
                             std::process::exit(101);
@@ -177,6 +174,9 @@ impl Worker {
                     None
                 };
 
+                // Deny critical system calls on sandbox lock.
+                let mut filter_crit = false;
+
                 // Create sentinel, that will handle graceful teardown.
                 let mut sentinel = Sentinel::new(&self);
 
@@ -184,19 +184,36 @@ impl Worker {
                 self.worker_data.increment_worker_total();
 
                 loop {
-                    // Confine and drop filter if sandbox is locked.
-                    if let Some(filter) = ctx {
-                        if Sandbox::locked_once() {
+                    // Confine thread if sandbox is locked.
+                    if Sandbox::locked_once() {
+                        if let Some(filter) = ctx {
                             // SAFETY: filter pointer is owned by the
                             // monitor thread and is valid for Syd's
                             // lifetime.
                             let error = unsafe { seccomp_load(filter as const_scmp_filter_ctx) };
+                            ctx = None; // Leak ctx intentionally.
 
                             // SAFETY: We use exit_group(2) here to bail,
                             // because this confinement is a critical safety feature.
                             if error != 0 {
                                 let errno = Errno::from_raw(error.abs());
-                                alert!("ctx": "boot", "op": "confine_emulator_thread",
+                                alert!("ctx": "boot", "op": "confine_emu_thread",
+                                    "msg": format!("failed to confine: {error}"),
+                                    "err": errno as i32);
+                                std::process::exit(101);
+                            }
+                        }
+
+                        if !filter_crit {
+                            filter_crit = true;
+
+                            // SAFETY: We use exit_group(2) here to bail,
+                            // because this confinement is a critical safety feature.
+                            if let Err(error) =
+                                confine_scmp(ScmpAction::KillProcess, EMU_LOCK_SYSCALLS)
+                            {
+                                let errno = error.errno().unwrap_or(Errno::ENOSYS);
+                                alert!("ctx": "boot", "op": "confine_emu_thread",
                                     "msg": format!("failed to confine: {error}"),
                                     "err": errno as i32);
                                 std::process::exit(101);
@@ -205,11 +222,9 @@ impl Worker {
                             let safe_setid = self.flags.intersects(
                                 Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID,
                             );
-                            info!("ctx": "confine", "op": "confine_emulator_thread",
+                            info!("ctx": "confine", "op": "confine_emu_thread",
                                 "msg": format!("emulator thread confined with{} SROP mitigation",
                                     if safe_setid { "out" } else { "" }));
-
-                            ctx = None;
                         }
                     }
 
@@ -264,7 +279,7 @@ impl Worker {
         // Use libc::seccomp_notif rather than libseccomp_sys's.
         // The latter is opaque and requires us to do a heap
         // allocation which we don't always want.
-        let mut req: MaybeUninit<libc::seccomp_notif> = MaybeUninit::zeroed();
+        let mut req: MaybeUninit<seccomp_notif> = MaybeUninit::zeroed();
 
         if let Some(timer) = timer {
             // SAFETY: Something is awfully wrong if we cannot
@@ -320,8 +335,6 @@ impl Worker {
             self.fd,
             Arc::clone(&self.cache),
             Arc::clone(&self.sandbox),
-            self.crypt_map.as_ref().map(Arc::clone),
-            Arc::clone(&self.unix_map),
         );
         let mut response = handler(request);
 
@@ -337,7 +350,7 @@ impl Worker {
             EIDRM if response.id == 0 && response.val == 0 && response.flags == 0 => return,
             EOWNERDEAD if response.id == 0 && response.val == 0 && response.flags == 0 => {
                 crate::warn!("ctx": "confine", "op": "enter_ghost_mode", "pid": req.pid,
-                    "sys": syscall, "arch": SydArch(req.data.arch), "args": req.data.args,
+                    "sys": syscall, "arch": SydArch::from(req.data.arch), "args": req.data.args,
                     "src": proc_mmap(req.pid(), req.data.instr_pointer).ok());
 
                 // Correct the pseudo errno back to success.
@@ -349,7 +362,7 @@ impl Worker {
             _ => false,
         };
 
-        let response = libc::seccomp_notif_resp {
+        let response = seccomp_notif_resp {
             id: response.id,
             val: response.val,
             error: response.error,
@@ -368,7 +381,7 @@ impl Worker {
             // Inform the monitor thread and other emulator threads to exit,
             // and notify the syd_aes thread as necessary.
             self.should_exit.store(true, Ordering::Relaxed);
-            if let Some(ref crypt_map) = self.crypt_map {
+            if let Some(ref crypt_map) = self.cache.crypt_map {
                 let (aes_map, cvar) = &**crypt_map;
                 let _aes_map = aes_map.lock().unwrap_or_else(|e| e.into_inner());
                 cvar.notify_one();
@@ -826,6 +839,54 @@ impl Worker {
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
+
+            // fchdir(2) may be used only with syscall argument cookies,
+            // and positive file descriptor arguments.
+            let sysname = "fchdir";
+            #[expect(clippy::useless_conversion)]
+            match ScmpSyscall::from_name(sysname) {
+                Ok(syscall) => {
+                    ctx.add_rule_conditional(
+                        ScmpAction::Allow,
+                        syscall,
+                        &[
+                            scmp_cmp!($arg0 <= RawFd::MAX as u64),
+                            scmp_cmp!($arg1 == (*FCHDIR_COOKIE_ARG1).into()),
+                            scmp_cmp!($arg2 == (*FCHDIR_COOKIE_ARG2).into()),
+                            scmp_cmp!($arg3 == (*FCHDIR_COOKIE_ARG3).into()),
+                            scmp_cmp!($arg4 == (*FCHDIR_COOKIE_ARG4).into()),
+                            scmp_cmp!($arg5 == (*FCHDIR_COOKIE_ARG5).into()),
+                        ],
+                    )?;
+                }
+                Err(_) => {
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
+                        "msg": format!("invalid or unsupported syscall {sysname}"));
+                }
+            }
+
+            // umask(2) may be used only with syscall argument cookies,
+            let sysname = "umask";
+            #[expect(clippy::useless_conversion)]
+            match ScmpSyscall::from_name(sysname) {
+                Ok(syscall) => {
+                    ctx.add_rule_conditional(
+                        ScmpAction::Allow,
+                        syscall,
+                        &[
+                            scmp_cmp!($arg1 == (*UMASK_COOKIE_ARG1).into()),
+                            scmp_cmp!($arg2 == (*UMASK_COOKIE_ARG2).into()),
+                            scmp_cmp!($arg3 == (*UMASK_COOKIE_ARG3).into()),
+                            scmp_cmp!($arg4 == (*UMASK_COOKIE_ARG4).into()),
+                            scmp_cmp!($arg5 == (*UMASK_COOKIE_ARG5).into()),
+                        ],
+                    )?;
+                }
+                Err(_) => {
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
+                        "msg": format!("invalid or unsupported syscall {sysname}"));
+                }
+            }
         } else {
             // trace/allow_unsafe_nocookie: Allow access without cookies.
             for sysname in [
@@ -839,6 +900,8 @@ impl Worker {
                 "unlinkat",
                 "openat2",
                 "uname",
+                "fchdir",
+                "umask",
             ] {
                 match ScmpSyscall::from_name(sysname) {
                     Ok(syscall) => {
@@ -1230,7 +1293,7 @@ impl<'a> Sentinel<'a> {
 
     #[expect(clippy::arithmetic_side_effects)]
     fn deny_syscall(&self, seccomp_id: u64, errno: Errno) {
-        let response = libc::seccomp_notif_resp {
+        let response = seccomp_notif_resp {
             id: seccomp_id,
             val: 0,
             error: -(errno as i32),
