@@ -48,6 +48,7 @@ use crate::{
     fs::{closeexcept, peer_cred},
     hash::SydHashMap,
     info,
+    landlock::Errata,
     landlock_policy::LandlockPolicy,
     path::{XPath, XPathBuf},
     retry::retry_on_eintr,
@@ -339,12 +340,11 @@ impl IpcWorker {
     ) -> SydResult<ScmpFilterContext> {
         if !dry_run {
             // SAFETY: Set up a Landlock sandbox to disallow all access.
-            // We do not enable scoped signals not to break inter-thread
-            // signaling. This was fixed in recent versions of Landlock.
-            // TODO: Make LandlockPolicy use scoped signals only if fixed.
             let abi = crate::landlock::ABI::new_current();
+            let errata = crate::landlock::Errata::query();
             let policy = LandlockPolicy {
                 scoped_abs: true,
+                scoped_sig: errata.contains(Errata::SCOPED_SIGNAL_SAME_TGID),
                 ..Default::default()
             };
             let _ = policy.restrict_self(abi);
